@@ -2,12 +2,12 @@
 # -*- coding: utf-8 -*-
 
 import ressources
-
 import sys
 import ConfigParser
 import string
 import random
 import datetime
+import subprocess
 from appdirs import *
 from argparse import ArgumentParser
 import MySQLdb as mysql
@@ -24,7 +24,6 @@ def query_yes_no(question, default="yes"):
         prompt = " [y/N] "
     else:
         raise ValueError("invalid default answer: '%s'" % default)
-
     while 1:
         sys.stdout.write(question + prompt)
         choice = raw_input().lower()
@@ -37,12 +36,13 @@ def query_yes_no(question, default="yes"):
                              "(or 'y' or 'n').\n")
 
 
-class mysqlrocket:
+class MySQLRocket:
 	name = "default"
 	host = "localhost"
 	user = "root"
 	port = 3306
 	password = ""
+	mysqldump = "/usr/bin/mysqldump"
 	config = ConfigParser.ConfigParser()
 	configfile = ""
 
@@ -60,7 +60,6 @@ class mysqlrocket:
 					self.config.write(configfile)
 					print '\nConfiguration file has been update: '+os.path.abspath(self.config_file)
 
-
 	def load(self, config_id):
 		if (self.config.has_section(config_id)):
 			try:
@@ -68,6 +67,7 @@ class mysqlrocket:
 				self.port=int(self.config.get(config_id, 'port', 0))
 				self.user=self.config.get(config_id, 'user', 0)
 				self.password=self.config.get(config_id, 'password', 0)
+				self.mysqldump=self.config.get(config_id, 'mysqldump', 0)
 			except ConfigParser.NoOptionError:
 				print 'Invalid or outdated config'
 				remove_config=query_yes_no("Do you want to remove invalid config")
@@ -80,6 +80,7 @@ class mysqlrocket:
 			input_port = raw_input("port ("+str(self.port)+")> ")
 			input_user = raw_input("user ("+self.user+")> ")
 			input_password = raw_input("pass > ")
+			input_mysqldump = raw_input("mysqldump absolute path ("+self.mysqldump+")> ")
 			save_config=query_yes_no("Do you want to save configuration?")
 			if (save_config):
 				self.config.add_section(config_id)
@@ -97,6 +98,10 @@ class mysqlrocket:
 				else:
 					self.config.set(config_id, 'user', self.user)
 				self.config.set(config_id, 'password', input_password)
+				if input_mysqldump:
+					self.config.set(config_id, 'mysqldump', input_mysqldump)
+				else:
+					self.config.set(config_id, 'mysqldump', self.mysqldump)
 				if not os.path.exists(os.path.dirname(self.config_file)):
 					os.makedirs(os.path.dirname(self.config_file))
 				with open(self.config_file, 'wb') as configfile:
@@ -110,19 +115,13 @@ class mysqlrocket:
 		if db_password=='':
 			db_password="".join([random.choice(dictionnary) for i in range(8)])
 		try:
-			# Establish MySQL connection
 			conn = mysql.connect(host=self.host, port=self.port, user=self.user, passwd=self.password)
 			cursor = conn.cursor()
-
-			# Execute database and user creation queries
 			cursor.execute('CREATE DATABASE {database_name};'.format(database_name=db_name))
 			cursor.execute("GRANT ALL ON {database_name}.* TO {username}@{host} IDENTIFIED BY '{password}'".format(database_name=db_name, username=db_user, host=self.host, password=db_password))
 			cursor.execute('FLUSH PRIVILEGES;')
-
-			# Clean up the connections
 			cursor.close()
 			conn.close()
-
 		except mysql.Error, e:	
 			print "Database creation fail"
 			print('Error %d: %s' % (e.args[0], e.args[1]))
@@ -138,29 +137,28 @@ class mysqlrocket:
 		print '##################################'
 
 	def ls(self, db_pattern='%'):
+		db_list=self.showdb(db_pattern)
+		print '##################################'
+		print '#        Database list           #'
+		print '##################################'
+		for database in db_list:
+			print '  '+database
+		print '##################################'		
+
+	def showdb(self, db_pattern='%'):
+		db_list=[]
 		try:
-			# Establish MySQL connection
 			conn = mysql.connect(host=self.host, port=self.port, user=self.user, passwd=self.password)
 			cursor = conn.cursor()
-
-			# Show databases
 			cursor.execute("SHOW DATABASES LIKE '{pattern}';".format(pattern=db_pattern))
-
-			print '##################################'
-			print '#        Database list           #'
-			print '##################################'
 			for database in cursor.fetchall():
-				print '  '+database[0]
-			print '##################################'
-
-			# Clean up the connections
+				db_list.append(database[0])
 			cursor.close()
 			conn.close()
-
 		except mysql.Error, e:
 			print('Error %d: %s' % (e.args[0], e.args[1]))
 			sys.exit(1)
-
+		return db_list
 
 	def rm(self, db_name):
 		print db_name
@@ -174,18 +172,12 @@ class mysqlrocket:
 			print '"'+db_name+'" is a protected database, you should not delete it'
 			exit()
 		try:
-			# Establish MySQL connection
 			conn = mysql.connect(host=self.host, port=self.port, user=self.user, passwd=self.password)
 			cursor = conn.cursor()
-
-			# Show databases
 			cursor.execute('DROP DATABASE {database_name};'.format(database_name=db_name))
 			cursor.execute('DROP USER {username}@{host};'.format(username=db_name, host=self.host))
-
-			# Clean up the connections
 			cursor.close()
 			conn.close()
-
 		except mysql.Error, e:
 			print('Error %d: %s' % (e.args[0], e.args[1]))
 			sys.exit(1)
@@ -193,35 +185,44 @@ class mysqlrocket:
 		print "Database was successfully deleted!"
 		print '##################################'
 
+	def dp(self, db_name):
+		datenow= datetime.datetime.now()
+		filename = db_name + "-" +datenow.strftime("%Y_%m_%d-%H_%M_%S") + ".gz"
+		try:
+			if self.password == "":
+				p1 = subprocess.Popen(self.mysqldump+" -u %s -h %s -e --opt --single-transaction --max_allowed_packet=512M -c %s" % (self.user, self.host, db_name), stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+			else:
+				p1 = subprocess.Popen("mysqldump -u %s -p%s -h %s -e --opt --single-transaction --max_allowed_packet=512M -c %s" % (self.user, self.password, self.host, db_name), stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+			p2 = subprocess.Popen("gzip -c > %s" % (filename), stdin=p1.stdout, shell=True)
+			p1.stdout.close()
+			output = p1.stderr.read()
+			if output == '':
+				print "The database has been dump to: "+filename
+			else:
+				print output
+				exit(1)
+		except subprocess.CalledProcessError as e:
+			print "Error: process exited with status %s" % e.returncode
+
 	def st(self, st_extended=False):
 		try:
-			# Establish MySQL connection
 			conn = mysql.connect(host=self.host, port=self.port, user=self.user, passwd=self.password)
 			cursor = conn.cursor()
-
 			print 'MySQL connection was successful!'
-
 			if st_extended:
-				# Show databases
 				cursor.execute('SHOW STATUS;')
-
 				print '##################################'
 				print '#       MySQL Server Status      #'
 				print '##################################'
 				for status in cursor.fetchall():
 					print '  '+status[0]+' = '+status[1]
 				print '##################################'
-
-			# Clean up the connections
 			cursor.close()
 			conn.close()
 
 		except mysql.Error, e:
 			print('Error %d: %s' % (e.args[0], e.args[1]))
 			sys.exit(1)
-		#your code goes here
-
-
 
 def launcher():
 	parser = ArgumentParser(description=ressources.__description__,prog="mysqlrocket")
@@ -234,7 +235,7 @@ def launcher():
 
 	subparsers = parser.add_subparsers(help='Avalaible commands')
 
-	session=mysqlrocket()
+	session=MySQLRocket()
 	session.load('Server1')
 
 	parser_mk = subparsers.add_parser('mk',description='Create a MySQL database', help='Create a MySQL database')
@@ -247,6 +248,12 @@ def launcher():
 	parser_rm = subparsers.add_parser('rm',description='Delete a MySQL database', help='Delete a MySQL database')
 	parser_rm.add_argument('rm_db_name', type=str, help='Name of the deleted database')
 
+	parser_dp = subparsers.add_parser('dp',description='Dump a MySQL database', help='Dump a MySQL database')
+	parser_dp.add_argument('dp_db_name', type=str, help='Name of the dumped database')
+
+	parser_bk = subparsers.add_parser('bk', description='Backup all databases on MySQL server', help='Backup all databases on MySQL server')
+	parser_bk.add_argument('bk_db_pattern', metavar='<backup_pattern>', type=str, nargs='?', default='%', help='Dump only databases name matching pattern')
+
 	parser_st = subparsers.add_parser('st',description='Check your mysqlrocket config file and MySQL server status', help='Check your mysqlrocket config file and MySQL server status')
 	parser_st.add_argument('st_extended', metavar='<status>', type=str, nargs='?', default='basic', help='Choose between "basic" or "full" status')
 
@@ -261,6 +268,14 @@ def launcher():
 	if hasattr(args,'rm_db_name'): 
 	    session.rm(args.rm_db_name)
 
+	if hasattr(args,'dp_db_name'): 
+	    session.dp(args.dp_db_name)
+
+	if hasattr(args,'bk_db_pattern'):
+		db_list=session.showdb(args.bk_db_pattern)
+		for database in db_list:
+	 		session.dp(database)
+
 	if hasattr(args,'st_extended'):
 		if args.st_extended=="full":
 			session.st(True)
